#!/usr/bin/evn python3
# Import Threading Module
import threading
import time
import math
import json
from datetime import datetime

from pymodbus.exceptions import ConnectionException, ModbusException

import emBRICK.binary_utils as binary_utils

# Import pymodbus for the Modbus RTU Module
from pymodbus.client.sync import ModbusSerialClient as ModbusClient
lock = threading.Lock()

filename = "error"
ERROR = open(filename, "w+")

brick_name = {2181: "CAE_G8Di8Do", 4602: "CAE_B3U4I", 4603: "CAE_B3U4I", 2470: "CAE_G2Mi2Ao", 2471: "CAE_G2Mi2Ao",
            2472: "CAE_G2Mi2Ao", 2431: "CAE_G4Ai4Tmp", 2432: "CAE_G4Ai4Tmp", 2433: "CAE_G4Ai4Tmp", 2434: "CAE_G4Ai4Tmp",
            2435: "CAE_G4Ai4Tmp", 2436: "CAE_G4Ai4Tmp"}

class local_master:
    ''' Sorting the information about the local master '''
    __slots__ = ['number_bricks','state','master_id','prot_ver','soft_ver', 'manu_id', 'res1', 'res2']
    def __init__(self, number_bricks, state, master_id, prot_ver, soft_ver, manu_id, res1, res2):
        self.number_bricks = number_bricks
        self.state = state
        self.master_id = master_id
        self.prot_ver = prot_ver
        self.soft_ver = soft_ver
        self.manu_id = manu_id
        self.res1 = res1
        self.res2 = res2


class slave_module:
    ''' Sorting the Information about the Bricks '''
    __slots__ = ['state', 'data_mosi', 'data_miso', 'prot_ver', 'hard_rev', 'brick_id', 'manu_id', 'offset_mosi', 'offset_miso', "name"]
    def __init__(self, state, data_mosi, data_miso, prot_ver, hard_rev, brick_id, manu_id, offset_mosi, offset_miso, name):
        self.state = state
        self.data_mosi = data_mosi
        self.data_miso = data_miso
        self.prot_ver = prot_ver
        self.hard_rev = hard_rev
        self.brick_id = brick_id
        self.manu_id = manu_id
        self.offset_mosi = offset_mosi
        self.offset_miso = offset_miso
        self.name = name


class connection:
    def __init__(self):
        self.port = ''
        self.baudrate = 460800
        self.number = 0
        self.slave_address = []
        self.unit_id = []

        self.node = ''
        self.timeout = 0.1
        self.updateRate = 0.005
        self.master = {}
        self.slave =  {}

    def start(self):
        print("---------------------------------------------------\n")
        print("emBRICK(R), Starterkit Modbus RTU Python\n")
        print("(c) 2022 by IMACS GmbH \n")
        print("---------------------------------------------------\n")
        # Connecting to the LWC's on the given Slave Adress
        self.node = ModbusClient(method='rtu', port=self.port, timeout= self.timeout, baudrate= self.baudrate)
        ERROR.write(f'Error Log\nCommunication over Modbus RTU with Python\
         Baudrate: {self.baudrate}, \tPort: {self.port}, \tTimeout: {self.timeout}\n\n')
        if self.node.is_socket_open() == True:
            self.node.close()
        connected = self.node.connect()
        if not connected:
            ERROR.write(f'{datetime.now()}\tERROR: Connection with Port {self.port} failed!')
            ERROR.close()
            exit()
        # Initialize the Functions to start the Program
        bB_update.getLWCsInfo()
        for id in self.unit_id:
            self.master["local" + str(id)] = local_master(bB_update.local_data[id][0], bB_update.local_data[id][1], ((bB_update.local_data[id][2] << 8) + bB_update.local_data[id][3]),
                        bB_update.local_data[id][4],bB_update.local_data[id][5], bB_update.local_data[id][6], bB_update.local_data[id][7], bB_update.local_data[id][8])                
            
            self.slave["master" + str(id)] = {}
            for i in range(self.master["local" + str(id)].number_bricks):
                found = brick_name.get((bB_update.local_data[id][9+(i*11)+5] << 8) + bB_update.local_data[id][9+(i*11)+6], "Not found!")
                self.slave["master" + str(id)]["Brick"+ str(i+1)] = slave_module(bB_update.local_data[id][9+(i*11)+0], bB_update.local_data[id][9+(i*11)+1], bB_update.local_data[id][9+(i*11)+2],
                        bB_update.local_data[id][9+(i*11)+3], bB_update.local_data[id][9+(i*11)+4], (bB_update.local_data[id][9+(i*11)+5] << 8) + bB_update.local_data[id][9+(i*11)+6],
                        (bB_update.local_data[id][9+(i*11)+7] << 8) + bB_update.local_data[id][9+(i*11)+8],\
                        bB_update.local_data[id][9+(i*11)+9], bB_update.local_data[id][9+(i*11)+10], found )
        bB_update.update_first()
        bB.createEmptyList()
        updatcycle = threading.Thread(target=bB_update.update)
        updatcycle.daemon = True
        updatcycle.start()
        init()
        #time.sleep(1)
        #t1.join()

connect = connection()


class brickBus_communication:
    #
    def __init__(self):
        self.local_data = {}
        self.updated = {}
        self.updates = {}
        self.stopThread = False
    def getLWCsInfo(self):
        for id in connect.unit_id:
            i = 0
            # Reads in the Init Data in the address range 1000h ... 107 ch
            while(i < 5):
                response = connect.node.read_input_registers(4096, 104, unit=id)
                if not response.isError():
                    data_16byte = response.registers
                    self.local_data[id] = binary_utils.convert16bitTo8bitList(data_16byte)
                    break
                else:
                    ERROR.write(f'{datetime.now()}\tERROR: Connection to the Coupling-Master with Modbus Address {id} failed!\n')
                    i += 1
                if i == 5:
                    ERROR.write(f'{datetime.now()}\tERROR: Connection to the Coupling-Master with Modbus Address {id} failed!\n')
                    connect.unit_id.remove(id)
        if len(connect.unit_id) == 0:
            print("Connection failed.")
            ERROR.close()
            exit()

    def update_first(self, ):
        ''' Update the emBricks in the individual updaterate '''
        for id in connect.unit_id:
            i = 0
            while(i < 5):
                response = connect.node.read_input_registers(0, 120, unit=id)
                time.sleep(0.005)
                if not response.isError():
                    self.updates[id] = response.registers
                else:
                    ERROR.write(f'{datetime.now()}\tERROR: Checksum Error or Timeout. {response}\n')
                    i += 1
                if self.updates[id] != 0:
                    self.updated[id] = binary_utils.convert16bitTo8bitList(self.updates[id])
                    break
            if i == 5:
                ERROR.write(f'{datetime.now()}\tERROR: Connection with LWCs on Modbus Address {id} not possible\n')
                connect.unit_id.remove(id)
        if len(connect.unit_id) == 0:
            ERROR.close()
            exit()


    def update(self):
        # Update the emBricks in the individual updaterate
        while True:
            if not connect.node.is_socket_open() == True:
                connect.node.connect()
            for id in connect.unit_id:
                if not bB.put[id] == []:
                    with lock:
                        bB.set[id] = binary_utils.convert8bitTo16bitList(bB.put[id])
                arguments = {
                    'read_address': 0,
                    'read_count': bB.buffer_length[id],
                    'write_address': 0,
                    'write_registers': bB.set[id],
                }

                response = connect.node.readwrite_registers(unit=id, **arguments)
                if not response.isError():
                    self.updates[id] = response.registers
                else:
                    ERROR.write(f'{datetime.now()}\tERROR: Connection error: {response}\n')

                if self.updates[id] != 0:
                    lock.acquire()
                    self.updated[id] = binary_utils.convert16bitTo8bitList(self.updates[id])
                