# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 12:18:08 2023

@author: jkris
"""

from re import findall
import logging
import doq  # pylint: disable=W0611
from . import helper as ch


def run_doq(pyfilepath: str, formatter: str = "numpy", write: bool = True):
    """Auto-Generate Docstrings with doq (py-doq) package. Doq command is
    executed via subprocess command to create docstrings where missing.

    Parameters
    ----------
    pyfilepath : str
        Full path to python (.py) file
    formatter : str
        ["numpy","google","sphinx"] : See doq documentation for formatting options
    write : bool
        True = write the auto-gen docstrings in-place

    Returns
    -------
    str
        Summary of commandoutputs
    """
    # Generate Simple Docstrings with doq where they dont exists
    doq_out, doq_err = ch.run_capture_out(
        ["dq", "-f", pyfilepath, f"--formatter={formatter}"]
    )
    if (len(doq_out) + len(doq_err)) == 0:
        return ""
    if write:
        doq_out, doq_err = ch.run_capture_out(
            ["doq", "-f", pyfilepath, "-w", f"--formatter={formatter}"]
        )
        if len(doq_err) == 0:
            doq_str = f"{ch.format_header('Doq Output')}\n\n\
                Simple Docstrings Added. Please Complete Them!\n\n"
        else:
            doq_str = f"{ch.format_header('Doq Output')}\n{doq_err}\n"
    else:
        results = findall(r'("""(.|\n|\r)*?""")', doq_out + doq_err)
        doq_strings = "\n".join([result[0] for result in results])
        do