import inspect
from inspect import currentframe, getmembers
from datetime import datetime
from telebot import TeleBot
from telebot.types import ForceReply
import json
import os

from .utils import History, get_sender
from .api import ApiManager

from .keyboards import Button, KeyBoard, NavBar


class Bot:

    def __init__(self, api_key: str, history_file: str = "history.json", home_button_function: callable = None):
        assert history_file.endswith('.json'), "History file have to be a json extension file"
        self.name = 'Script'
        self.main_path = os.path.split(inspect.stack()[1][1])[0]
        self.bot = TeleBot(api_key)
        self.conf = {
            "telegram_api": "http://192.168.1.190:8519",
            "user_agent": "Connor Lil Homer was here"
        }
        assert self.conf.get("telegram_api"), "'telegram_api' key missing in config.json file"
        self.api = ApiManager(self.conf.get("telegram_api"))
        self.bot_whitelist = self.api.get_allowed_users()
        self.functions_doc = dict()
        self.modules = dict()
        self.modules_obj = dict()
        self.modules_callbacks = dict()
        self.history: History = History(history_file)
        self.reply_callbacks = dict()
        self.father_bot = self.add_module("Father Bot", FatherBot, home_button_function=home_button_function)

    def __notify_admins__(self, function_name: str, chat_id: int):
        for admin in self.bot_whitelist:
            self.bot.send_message(
                admin,
                f"A user with ID {chat_id} tried to talk the bot to {function_name}\
                function without permissions"
            )

    def __multi_handler__(self, message):
        chat_id, msg_command, msg_text = self.__prepare_args_from_received_msg__(message)
        if message.reply_to_message:
            for callback, callback_content in self.reply_callbacks.items():
                if message.reply_to_message.id == callback:
                    reply_callback, reply_append_command = s